<?php
/*

Copyright (c)2002-2003 DuckCorp(tm) and RtpNet(tm)



This file is part of LMS.

LMS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

LMS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LMS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/**
* Host Type : unknown
*
* @const LMS_HTYPE_UNKNOWN
*/
define('LMS_HTYPE_UNKNOWN', 0);
/**
* Host Type : common
*
* @const LMS_HTYPE_COMMON
*/
define('LMS_HTYPE_COMMON', 1);
/**
* Host Type : server
*
* @const LMS_HTYPE_SERVER
*/
define('LMS_HTYPE_SERVER', 2);

/**
* LMS Host Manager CLass
*
* @access	public
*/
class LmsHostManager extends LmsObjectManager
{

	/**
	* Constructor
	*
	* @access	public
	* @param	object DftkLdapConnection	&$ldap_connection
	*/
	function LmsHostManager(&$tracemgr, &$ldap_connection)
	{
		global		$config;

		LmsObjectManager::LmsObjectManager(&$tracemgr, &$ldap_connection);

		$this->_dn_attr = "cn";
		$this->_root_dn = "ou=Hosts,ou=Groups,".$config[base];
		$this->_propagation_key_list = array('hostname', 'servername', 'primaryServername');
		$this->_check_key_list = array();
		$this->_base_objectclasses = array('top');

		$this->_register_attr("Host Name", "cn", LMS_ATYPE_STR, 30, 1, true);
		$this->_register_attr("MAC Adresses", "macaddress", LMS_ATYPE_MAC, null, 10, true);
		$this->_register_attr("IP Adresses", "iphostnumber", LMS_ATYPE_IP, null, 10, true);
		$this->_register_attr("DNS Aliases", "dnsalias", LMS_ATYPE_STR, 30, 1, false);
		$this->_register_attr("Is a Router", "isrouter", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Is a mobile computer", "ismobilecomputer", LMS_ATYPE_BOOL, null, 1, false);
		$this->_register_attr("Owner", "owneruid", LMS_ATYPE_STR, 15, 1, true);
		$this->_register_attr("Administrator", "adminuid", LMS_ATYPE_STR, 15, 1, false);
		$this->_register_attr("Description", "description", LMS_ATYPE_STR, 300, 1, false);
		
		register_shutdown_function(array(&$this, "_LmsHostManager"));
	}

	/**
	* Destructor
	*
	* @access	public
	*/
	function _LmsHostManager()
	{
	}

	/**
	* Give Host Type
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_host_type()
	{
		$r =& $this->_tracemgr->create_trace();
		
		if (!is_array($this->_attrs['objectclass']['value']))
			$r->add_event('lms-api_hosttypenou');
		elseif (in_array('lmsCommonHost', $this->_attrs['objectclass']))
			$r->set_result('host_type', LMS_HTYPE_COMMON);
		elseif (in_array('lmsServerHost', $this->_attrs['objectclass']))
			$r->set_result('host_type', LMS_HTYPE_SERVER);
		else
			$r->add_event('lms-api_hosttypebadt');

		return $r;
	}

	/**
	* Find Host
	*
	* @access	public
	* @param	string			$cn			CN of the Host
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &find($cn)
	{
		$r =& $this->_lc->search($this->_root_dn, "cn=".$cn, $this->_ent);
		if ($r->has_error)
			$r->add_event('lms-api_hostfindse', $gidnumber);
		else
		{
			$nb = $r->get_result('count');

			if ($nb!=0)
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
			else
				$r->add_event('lms-api_hostfindnf', $cn);

			if ($n>0)
				$r->add_event('lms-api_probconsit');
		}

		return $r;
	}

	/**
	* Get a Host list
	*
	* Always fetched attributs : objectclass, cn
	*
	* @access	public
	* @param	integer			$c			Host Type to list (0 => every hosts)
	* @param	array			$attr			Complementary attributs to fetch
	* @param	string			$crit			Alternative criterias
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_list($c, $attr = array(), $crit = "")
	{
		if (!is_array($attr))
			$attr = array();
		$attr = array_merge(array('objectclass', 'cn'), $attr);
		if (!$crit)
			$crit = "cn=*";
		$badt = false;
		switch ($c)
		{
		case LMS_HTYPE_UNKNOWN:
			$r =& $this->_lc->search($this->_root_dn, $crit, $this->_ent, $attr);
			break;
		case LMS_HTYPE_COMMON:
			$r =& $this->_lc->search("ou=CommonHosts,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		case LMS_HTYPE_SERVER:
			$r =& $this->_lc->search("ou=ServerHosts,".$this->_root_dn, $crit, $this->_ent, $attr, false);
			break;
		default:
			$bast = true;
			$r =& $this->_tracemgr->create_trace();
			$r->add_event('lms-api_hostlistbadt', $crit);
		}

		if (!badt)
		{
			if ($r->has_error())
				 $r->add_event('lms-api_hostlistse', $crit);
			else
			{
				$this->_entry_to_attrs(0, &$r);
				$r->set_result('count', $this->_ent_get_count());
			}
		}

		return $r;
	}

	/**
	* Additionnal Check (OVERLOAD)
	*
	* @access	private
	* @return	boolean		$ok		True if successful
	*/
	function _check_changes($mod = array())
	{
		$r =& LmsObjectManager::_check_changes($mod);

		$r2 =& $this->get_host_type();
		$r->merge_traces($r2);
		if (!$r2->has_error())
		{
			$host_type = $r2->get_result('host_type');
			if ($host_type==LMS_HTYPE_SERVER && !$this->_attrs['adminuid']['value'])
				$r->add_event('lms-api_hostchkchgmv');
		}

		return $r;
	}

	/**
	* Return a sub DN if needed (OVERLOAD)
	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		$r2 =& $this->get_host_type();
		$host_type = $r2->get_result('host_type');

		switch($host_type)
		{
		case LMS_HTYPE_SERVER:
			return "ou=ServerHosts";
		default:
			return "ou=CommonHosts";
		}
	}

	/**
	* Change Host Type
	*
	* @access	public
	* @param	integer			$new_host_type		Type of Host
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &set_host_type($new_host_type)
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_ent_get_count() != 0)
		{
			$r2 =& $this->get_host_type();
			$r->merge_traces($r2);
			if ($r->has->error())
				$r->add_event('lms-api_hostsettse');
			else
			{
				$host_type = $r->get_result('host_type');

				if ($new_host_type==$host_type)
					$r->add_event('lms-api_hostsettntd');
			}
		}

		if (!$r->has_error())
		{
			$r2 =& $this->_set_host_type($new_host_type);
			$r->merge_traces($r2);
		}

		return $r;
	}

	/**
	* Change Host Type (internal use)
	*
	* @access	private
	* @param	integer			$new_host_type		Type of Host
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_set_host_type($new_host_type = null, $set_changed = true)
	{
		$r =& $this->_tracemgr->create_trace();

		if (!$new_host_type)
			$r =& $this->get_host_type();
			$new_host_type = $r->get_result('host_type');

		switch($new_host_type)
		{
		case LMS_HTYPE_SERVER:
			$this->_attrs['objectclass']['value'] = array('lmsServerHost');
			$this->_activate_attrs(array('adminuid'));
			break;
		case LMS_HTYPE_COMMON:
			$this->_attrs['objectclass']['value'] = array('lmsCommonHost');
			$this->_desactivate_attrs(array('adminuid'));
			break;
		default:
			$r->add_event('lms-api_hostsettbadt');
		}

		if ($set_changed)
			$this->_attrs['objectclass']['changed'] = true;

		return $r;
	}

	/**
	* Do Object need a renaming before saving changes ? (OVERLOAD)
	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return $this->_attrs['objectclass']['changed'];
	}

	/**
	* Convert an LDAP Entry to Attributs (OVERLOAD)
	*
	* @access	private
	* @param	integer		$i		Number of the Entry to convert
	*/
	function _entry_to_attrs($i, $r = null)
	{
		LmsObjectManager::_entry_to_attrs($i, &$r);

		$this->_set_host_type(null, false);
	}

}

?>
