<?php
//
// LMS-API version 0,1,0
//
// Copyright 2002 DuckCorp and RtpNet
//
// This program is free software ; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation (http://www.gnu.org/).
//



/**
* Number of Attributs types
*
* @const LMS_MAX_ATYPE
*/
define ('LMS_MAX_ATYPE', 6);

/**
* Attribut type : unknown
*
* @const LMS_ATYPE_UNKNOWN
*/
define ('LMS_ATYPE_UNKNOWN', 0);
/**
* Attribut type : integer
*
* @const LMS_ATYPE_INT
*/
define ('LMS_ATYPE_INT', 1);
/**
* Attribut type : string
*
* @const LMS_ATYPE_STR
*/
define ('LMS_ATYPE_STR', 2);
/**
* Attribut type : boolean
*
* @const LMS_ATYPE_BOOL
*/
define ('LMS_ATYPE_BOOL', 3);
/**
* Attribut type : telephone number
*
* @const LMS_ATYPE_TELNO
*/
define ('LMS_ATYPE_TELNO', 4);
/**
* Attribut type : mac address
*
* @const LMS_ATYPE_MAC
*/
define ('LMS_ATYPE_MAC', 5);
/**
* Attribut type : ip address
*
* @const LMS_ATYPE_IP
*/
define ('LMS_ATYPE_IP', 6);

/**
* LMS Object Base Class
*
* @access	private
*/
class LmsObjectManager
{
	/**
	* Result Manager
	*
	* @access	private
	* @var		string
	*/
	var	$_tracemgr;
	/**
	* List of all Object Attributes (with attributes parameters)
	*
	* @access	private
	* @var		array
	*/
	var	$_attrs;
	/**
	* LDAP Connection
	*
	* @access	private
	* @var		object DftkLdapConnection
	*/
	var	$_lc;
	/**
	* LDAP Entries
	*
	* @access	private
	* @var		object DftkLdapEntries
	*/
	var	$_ent;
	/**
	* DN of ths selected result or "" if a new object
	*
	* @access	private
	* @var		string
	*/
	var	$_dn;
	/**
	* Number of the Result currently used
	*
	* @access	private
	* @var		integer
	*/
	var	$_pos;
	/**
	* Attribute used to create the DN
	*
	* @access	private
	* @var		string
	*/
	var	$_dn_attr;
	/**
	* Root of this sort of Objects
	*
	* @access	private
	* @var		string
	*/
	var	$_root_dn;
	/**
	* List of keys in which to propagate Object renaming
	*
	* @access	private
	* @var		array
	*/
	var	$_propagation_key_list;
	/**
	* List of keys needed for check
	*
	* @access	private
	* @var		array
	*/
	var	$_check_key_list;
	/**
	* Base Objectclasses
	*
	* @access	private
	* @var		array
	*/
	var	$_base_objectclasses;

	/**
	* Constructor
	*
	* @access	public
	* @param	object DftkLdapConnection	&$ldap_connection
	*/
	function LmsObjectManager(&$tracemgr, &$ldap_connection)
	{
		$this->_tracemgr =& $tracemgr;

		if (!$this->_tracemgr->is_module("LMS-API"))
			$this->_tracemgr->register_traces("LMS-API",LMS_ROOT_PATH."/traces/");

		$this->_attrs = array();
		$this->_lc = $ldap_connection;
		$this->_ent =& new DftkLdapEntries(&$this->_tracemgr);
		$this->_dn = "";
		$this->_pos = 0;
		$this->_propagation_key_list = array();
		$this->_check_key_list = array();

		$this->_register_attr("Object Class", "objectclass", LMS_ATYPE_STR, 30, 15, true, false);

		register_shutdown_function(array(&$this, "_LmsObjectManager"));
	}

	/**
	* Destructor
	*
	* @access	private
	*/
	function _LmsObjectManager()
	{
		unset($this->_ent);
	}

	/**
	* Give DN of the current result
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_dn()
	{
		$r =& $this->_tracemgr->create_trace();

		if (!$this->_dn)
			$r->add_event('lms-api_objgetdnno');
		else
			$r->set_result('dn', $this->_dn);
			
		return $r;
	}

	/**
	* Set Name change of the current result (for rename remembering purpose)
	*
	* @access	private
	* @param	string			$new_dn			New DN
	*/
	function _set_name_change($new_dn)
	{
		$this->_dn = $new_dn;
		$this->_ent->set_dn($this->_pos, $new_dn);
	}

	/**
	* Register a new Attribut for this object
	*
	* @access	private
	* @param	string			$name			Human name of this Attribut
	* @param	string			$key			Key to access this Attribut
	* @param	string			$type			Value-type of this Attribut
	* @param	integer			$maxlength		Max length of this Attribut (for string values for eg)
	* @param	integer			$maxvalues		Maximum number of values for this Attribut
	* @param	boolean			$required		Is this Attribut required ?
	* @param	boolean			$changeable		Is this Attribut changeable ?
	* @return	boolean			$ok			True if successful
	*/
	function _register_attr($name, $key, $type, $maxlength, $maxvalues, $required = false, $changeable = true)
	{
		if ($type > LMS_MAX_ATYPE)
			return false;
		if (!is_bool($required))
			return false;
		if (!is_bool($changeable))
			return false;

		$this->_attrs[$key]['value'] = "";
		$this->_attrs[$key]['oldvalue'] = "";
		$this->_attrs[$key]['name'] = $name;
		$this->_attrs[$key]['type'] = $type;
		$this->_attrs[$key]['maxlenghth'] = $maxlength;
		$this->_attrs[$key]['maxvalues'] = $maxvalues;
		$this->_attrs[$key]['required'] = $required;
		$this->_attrs[$key]['changeable'] = $changeable;
		$this->_attrs[$key]['changed'] = false;

		return true;
	}

	/**
	* Give Attribut info
	*
	* @access	public
	* @param	string			$key			Key of the Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_attr_info($key)
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_attrs[$key])
		{
			$r->set_result('value', $this->_attrs[$key]['value']);
			$r->set_result('oldvalue', $this->_attrs[$key]['oldvalue']);
			$r->set_result('name', $this->_attrs[$key]['name']);
			$r->set_result('type', $this->_attrs[$key]['type']);
			$r->set_result('maxvalues', $this->_attrs[$key]['maxvalues']);
			$r->set_result('maxlength', $this->_attrs[$key]['maxlength']);
			$r->set_result('required', $this->_attrs[$key]['required']);
			$r->set_result('changeable', $this->_attrs[$key]['changeable']);
			$r->set_result('changed', $this->_attrs[$key]['changed']);
		}
		else
			$r->add_event('lms-api_objgetattrinfobadk', $key);
			
		return $r;
	}

	/**
	* Give values of Attributs
	*
	* @access	public
	* @param	string			$key			Key of the Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_attrs_value($tab)
	{
		$r =& $this->_tracemgr->create_trace();

		if (!is_array($tab))
			$tab = array($tab);

		foreach ($tab as $e)
		{
			if ($this->_attrs[$e])
				$r->set_result($e, $this->_attrs[$e]['value']);
			else
				$r->add_event('lms-api_objgetattrsvalbadk', $key);
		}
			
		return $r;
	}
	/**
	* Set value of an Attribut
	*
	* @access	public
	* @param	string			$key			Key of the Attribut
	* @param	mixed			$value			Value of the Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &set_attr_value($key, $value)
	{
		$r =& $this->_tracemgr->create_trace();

		if (!$this->_attrs[$key])
			$r->add_event('lms-api_objsetattrvalbadk', $key);
		if (!$this->_attrs[$key]['changeable'])
			$r->add_event('lms-api_objsetattrvalnc', $key);

		if (!$r->has_error())
		{
			$r2 =& $this->_set_attr_value($key, $value);
			$r->merge_traces($r2);
		}

		return $r;
	}


	/**
	* Wrapper to get Entries count
	*
	* @access	private
	* @return	integer			$nb			Number of entries
	*/
	function _ent_get_count()
	{
		$r =& $this->_ent->get_count();

		return $r->get_result('count');
	}

	/**
	* Set value of an Attribut (Internal)
	*
	* @access	private
	* @param	string			$key			Key of the Attribut
	* @param	mixed			$value			Value of the Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_set_attr_value($key, $value)
	{
		$r =& $this->_tracemgr->create_trace();

		if (is_array($value))
			for ($i=0; $i<count($value); $i++)
				$value[$i] = stripslashes($value[$i]);
		else
			$value = stripslashes($value);

		if ($this->_check_attr_value($key, $value))
		{
			if (($this->_attrs[$key]['maxvalues'] == 1) && (is_array($value)))
				$r->add_event('lms-api_objsetattrvalns', $key);
			elseif (($this->_attrs[$key]['maxvalues'] > 1) && (!is_array($value)))
				$r->add_event('lms-api_objsetattrvalnm', $key);

			if (!$r->has_error())
			{
				if ($this->_attrs[$key]['value'] != $value)
					$this->_attrs[$key]['changed'] = true;
				if (!$this->_attrs[$key]['changed'])
					$this->_attrs[$key]['oldvalue'] = $this->_attrs[$key]['value'];
				$this->_attrs[$key]['value'] = $value;
			}
		}
		else
			$r->add_event('lms-api_objsetattrvalbadf', $key);

		return $r;
	}

	/**
	* Check value before setting an Attribut
	*
	* @access	private
	* @param	string			$key			Key of the Attribut
	* @param	mixed			$value			Value of the Attribut
	* @return	boolean			$ok			True if successful
	*/
	function _check_attr_value($key, $value)
	{
		if (!is_array($value))
			$value = array($value);

		for ($i=0; $i<count($value); $i++)
		{
			$val = $value[$i];
			
			if ($val==null)
				continue;

			switch ($this->_attrs[$key]['type'])
			{
			case LMS_ATYPE_INT:
				if (!is_numeric($val))
					return false;
				break;
			case LMS_ATYPE_STR:
				if (!is_string($val))
					return false;
				break;
			case LMS_ATYPE_BOOL:
				if (!is_bool($val))
					return false;
				break;
			case LMS_ATYPE_TELNO:
				// International Telephone Number only
				if (preg_match("/^\+\d{8,15}$/", $val)<=0)
					return false;
				break;
			case LMS_ATYPE_MAC:
				if (preg_match("/^[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}:[0-9a-z]{2}$/i", $val)<=0)
					return false;
				break;
			case LMS_ATYPE_IP:
				if (preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i", $val)<=0)
					return false;
				break;
			default:
				return false;
			}
		}

		return true;
	}

	/**
	* Convert an LDAP Entry to Attributs
	*
	* @access	private
	* @param	integer			$i			Number of the Entry to convert
	*/
	function _entry_to_attrs($i = 0, $r = null)
	{
		if (($i<0) || ($i >= $this->_ent_get_count()))
			$i = 0;
		$r2 =& $this->_ent->get_entry($i);
		$tab =& $r2->get_result('entry');

		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['maxvalues'] == 1)
				$this->_attrs[$key]['value'] = $tab[$key][0];
			else
				$this->_attrs[$key]['value'] = $tab[$key];
			$this->_attrs[$key]['oldvalue'] = $this->_attrs[$key]['value'];
		}

		$r2 =& $this->_ent->get_dn($i);
		$this->_dn = $r2->get_result('dn');
		$this->_pos = $i;

		if ($r)
		{
			foreach ($this->_attrs as $key => $val)
				$r->set_result($key, $this->_attrs[$key]['value']);
		}
	}

	/**
	* Activete Attributes (used for Dynamic Objects)
	*
	* @access	private
	* @param	array			$tab			List of Attributs keys
	*/
	function _activate_attrs($tab, $do_clean = false)
	{
		foreach ($tab as $key)
		{
			$this->_attrs[$key]['changeable'] = true;
			if ($do_clean)
				$this->_attrs[$key]['value'] = "";
		}
	}

	/**
	* Desactivete Attributes (used for Dynamic Objects)
	*
	* @access	private
	* @param	array			$tab			List of Attributs keys
	*/
	function _desactivate_attrs($tab)
	{
		foreach ($tab as $key)
		{
			$this->_attrs[$key]['changeable'] = false;
			$this->_attrs[$key]['value'] = null;
		}
	}

	/**
	* Propagate Attributs changes into an LDAP Entry
	*
	* @access	private
	* @return	array			$mod			List of changed Attributs
	*/
	function _attrs_changes_to_current_entry()
	{
		$mod = array();
		$r2 =& $this->_ent->get_entry($this->_pos);
		$tab =& $r2->get_result('entry');
		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['value']==null)
			{
				unset($tab[$key][0]);
				if ($this->_attrs[$key]['changed'])
					$mod[] = $key;
			}
			elseif ($this->_attrs[$key]['changed'])
			{
				$mod[] = $key;
				if ($this->_attrs[$key]['maxvalues']>1)
					$tab[$key] = $this->_attrs[$key]['value'];
				else
					$tab[$key][0] = $this->_attrs[$key]['value'];
			}
		}
		return $mod;
	}

	/**
	* Has Attributs changed
	*
	* @access	private
	* @return	array			$mod			List of changed Attributs
	*/
	function _attrs_changed()
	{
		$mod = array();
		foreach ($this->_attrs as $key => $val)
		{
			if ($this->_attrs[$key]['changed'])
				$mod[] = $key;
		}
		return $mod;
	}

	/**
	* Clear all the Attributs
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &clear()
	{
		$r =& $this->_tracemgr->create_trace();

		foreach ($this->_attrs as $key => $val)
		{
			$this->_attrs[$key]['oldvalue'] = null;
			$this->_attrs[$key]['value'] = null;
			$this->_attrs[$key]['changed'] = false;
		}
		$this->_dn = "";
		$this->_pos = 0;
		$this->_ent->clear();

		return $r;
	}

  	/**
	* Display an Object content, for debugging purpose
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &display_object()
	{
		$r =& $this->_tracemgr->create_trace();
		
		dftk_display_array($this->_attrs);

		return $r;
	}

	/**
	* Select a result
	*
	* @access	public
	* @param	integer			$i			Number of the result to select
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &select_result($i)
	{
		$r =& $this->_tracemgr->create_trace();

		if (($i<0) || ($i >= $this->_ent_get_count()))
			$r->add_event('lms-api_objselresbadr', $i, $this->_ent_get_count());
		else
			$this->_entry_to_attrs($i, &$r);

		return $r;
	}

	/**
	* Give the number of results
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_result_number()
	{
		$r =& $this->_tracemgr->create_trace();

		$r->set_result('count', $this->_ent_get_count());

		return $r;
	}

	/**
	* Give the number of ths selected result
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &get_current_pos()
	{
		$r =& $this->_tracemgr->create_trace();

		$r->set_result('pos', $this->_pos);

		return $r;
	}

	/**
	* Select next result
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &next_result()
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_pos >= $this->_ent_get_count())
			$r->add_event('lms-api_objnextresnr', $this->_ent_get_count());
		else
			$this->select_result($this->_pos+1);

		return $r;
	}

	/**
	* Select previous result
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &previous_result()
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_pos <= 0)
			$r->add_event('lms-api_objprevresnr', $this->_ent_get_count());
		else
			$this->select_result($this->_pos-1);

		return $r;
	}

	/**
	* Sort results
	*
	* @access	public
	* @param	string			$attr1			First key for sorting
	* @param	string			$attr2			Second key for sorting
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &sort_result_by_attr($attr1, $attr2 = null)
	{
		$r =& $this->_tracemgr->create_trace();
		
		if (!$this->_attrs[$attr1])
			$r->add_event('lms-api_objsortresbada', $attr1);
		if ($attr2 != null)
			if (!$this->_attrs[$attr2])
				$r->add_event('lms-api_objsortresbada', $attr2);

		if (!$r->has_error())
			$this->_ent->sort_entries_by_attr($attr1, $attr2);

		return $r;
	}

	/**
	* Delete an Object
	*
	* @access	public
	* @param	string			$name			Name of Oject to delete
	* @return	boolean			$ok			True if successful
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &delete($name = "")
	{
		$r =& $this->_tracemgr->create_trace();
		
		if ($name)
		{
			$r2 =& $this->_lc->search($this->_root_dn, $this->_dn_attr."=".$name, $this->_ent, array($this->_dn_attr));
			$r->merge_traces($r2);
			if (!$r->has_error())
			{
				if ($this->_ent_get_count()<=0)
					$r->add_event('lms-api_objdelnf', $name);
				else
				{
					$this->_entry_to_attrs(0);
					$object = $this->_dn;
				}
			}
			else
				$r->add_event('lms-api_objdelse1', $name);
		}
		else
		{
			$object = $this->_dn;
			if (!$object)
				$r->add_event('lms-api_objdelno');
		}
		
		if (!$r->has_error())
		{
			$r2 =& $this->_lc->delete($object);
			$r->merge_traces($r2);
			if ($r->has_error())
				$r->add_event('lms-api_objdelse2');
		}

		return $r;
	}

	/**
	* Rename an Object
	*
	* @access	public
	* @param	string			$new_name		New name of Oject to delete
	* @param	string			$name			Name of Oject to delete
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &rename($name, $new_name)
	{
		$r =& $this->_tracemgr->create_trace();

		if (!$name && !$new_name)
			$r->add_event('lms-api_objrennn1');
		elseif (!$name)
			$r->add_event('lms-api_objrennn2');
		elseif (!$new_name)
			$r->add_event('lms-api_objrennn3');

		if (!$r->has_error())
		{
			$r2 =& $this->_lc->search($this->_root_dn, $this->_dn_attr."=".$name, $this->_ent, $this->_check_key_list);
			$r->merge_traces($r2);
			if (!$r->has_error())
			{
				if ($this->_ent_get_count()==0)
					$r->add_event('lms-api_objrennf');
				else
				{
					$this->_entry_to_attrs(0);
	
					$this->_set_attr_value($this->_dn_attr, $new_name);
	
					$r2 =& $this->commit_changes();
					$r->merge_traces($r2);
					if ($r->has_error())
						$r->add_event('lms-api_objrense');
				}
			}
			else
				$r->add_event('lms-api_objrense');
		}

		return $r;
	}

	/**
	* Save the changes in LDAP Database
	*
	* @access	public
	* @return	boolean			$ok			True if successful
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &commit_changes()
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_ent_get_count() == 0)
			$r->add_event('lms-api_objcomno');
		else
		{
			$mod = $this->_attrs_changed();
			if (count($mod)==0)
				$r->add_event('lms-api_objcomntd1');
			else
				$this->_pre_commit_changes();
		}

		if (!$r->has_warning() && !$r->has_error() && $this->_attrs['objectclass']['changed'])
		{
			$r2 =& $this->_fetch_all_attrs();
			$r->merge_traces($r2);
			if ($r->has_error())
				$r->add_event('lms-api_objcomse');
		}

		if (!$r->has_warning() && !$r->has_error())
		{
			$mod = $this->_attrs_changes_to_current_entry();
			if (count($mod) == 0)
				$r->add_event('lms-api_objcomntd2');
			else
			{
				$r2 =& $this->_check_changes($mod);
				$r->merge_traces($r2);
				if ($r->has_error())
					$r->add_event('lms-api_objcomse');
			}
		}
		
		if (!$r->has_warning() && !$r->has_error())
		{
			if ($this->_attrs['objectclass']['changed'])
				$r2 =& $this->_lc->modify($this->_dn ,$this->_ent, $this->_pos);
			else
				$r2 =& $this->_lc->modify($this->_dn ,$this->_ent, $this->_pos, $mod);
			$r->merge_traces($r2);
			if ($r->has_error())
				$r->add_event('lms-api_objcomse');
			else
				$this->_reset_changed_flags();
		}

		if (!$r->has_warning() && !$r->has_error() && ($this->_attrs[$this->_dn_attr]['changed'] || $this->_need_renaming()))
		{
			$object = $this->_dn;
			$sub = $this->_get_sub_dn();
			if ($sub)
				$sub.=",";
			$new_name = $this->_attrs[$this->_dn_attr]['value'];
			$new_object = $this->_dn_attr."=".strtolower($new_name).",".$sub.$this->_root_dn;
			if ($new_object != $object)
			{
				$r2 =& $this->_lc->rename($object, $new_object);
				$r->merge_traces($r2);
				if (!$r->has_error())
				{
					$this->_set_name_change($new_object);
					$name = $this->_attrs[$this->_dn_attr]['oldvalue'];
					$r2 =& $this->_propagate_object_renaming($name, $new_name);
					$r->merge_traces($r2);
					if ($r->has_error())
						$r->add_event('lms-api_objcompropse');
				}
				else
					$r->add_event('lms-api_objcomrense');
			}
		}

		return $r;
	}

	/**
	* Reset 'changed' flags of every Attribut
	*
	* @access	private
	*/
	function _reset_changed_flags()
	{
		foreach ($this->_attrs as $key => $val)
			$this->_attrs[$key]['changed'] = false;
	}

	/**
	* Propagate Key change
	*
	* @access	public
	* @param	string			$key			Key of the Attribut
	* @param	string			$old_value		Old value of Attribut
	* @param	string			$new_value		New value of Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &propagate_key_change($key, $old_value, $new_value)
	{
		$r =& $this->_tracemgr->create_trace();

		$ent =& new DftkLdapEntries(&$this->_tracemgr);

		$r2 =& $this->_lc->search($this->_dn, $key."=".$old_value, $ent);
		$r->merge_traces($r2);
		if (!$r->has_error())
		{
			$r3 =& $ent->get_count();
			$nb = $r3->get_result('count');
			if ($nb==0)
			{
				for ($i=0; $i<$nb; $i++)
				{
					$r2 =& $ent->get_entry($i);
					$tab =& $r2->get_result('entry');
					if (count($tab[$key])>1)
						$tab[key][array_search($old_value, $tab[$key])]=$new_value;
					else
						$tab[key][0]=$new_value;
						
					$r2 =& $this->_ent->get_dn($i);
					$dn = $r2->get_result('dn');

					$r2 =& $this->_lc->modify($dn ,$ent, $i, array($key));
					$r->merge_traces($r2);
					if ($r->has_error())
						$r->add_event('lms-api_objpropse2', $key, $old_value, $new_value, $dn);
					else
						$r->add_event('lms-api_objpropnot', $key, $old_value, $new_value, $dn);
				}
			}
		}
		else
			$r->add_event('lms-api_objpropse1', $key, $old_value, $new_value);

		return $r;
	}

	/**
	* Propagate Object_renaming
	*
	* @access	private
	* @param	string			$key			Key of the Attribut
	* @param	string			$old_value		Old value of Attribut
	* @param	string			$new_value		New value of Attribut
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_propagate_object_renaming($old_name, $new_name)
	{
		$r =& $this->_tracemgr->create_trace();
		
		foreach ($this->_propagation_key_list as $key)
		{
			$r2 =& $this->propagate_key_change($key, $old_name, $new_name);
			$r->merge_traces($r2);
		}

		return $r;
	}

	/**
	* Add a new Object in LDAP Database
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &add()
	{
		$r =& $this->_tracemgr->create_trace();

		$this->_pre_commit_changes();
		
		$mod = $this->_attrs_changes_to_current_entry();
		if (count($mod) == 0)
		{
			if (($this->_ent_get_count()==0))
				$r>add_event('lms-api_objaddbado1');
			else
				$r>add_event('lms-api_objaddbado2');
		}

		if (!$r->has_error())
		{
			$this->_attrs[objectclass][value] = array_merge($this->_attrs[objectclass][value], $this->_base_objectclasses);

			$r2 =& $this->_check_changes($mod);
			$r->merge_traces($r2);
			if ($r->has_error())
				$r>add_event('lms-api_objaddse');
		}
	
		if (!$r->has_error())
		{
			$sub = $this->_get_sub_dn();
			if ($sub)
				$sub.=",";
			$new_name = $this->_attrs[$this->_dn_attr]['value'];
			$new_object = $this->_dn_attr."=".strtolower($new_name).",".$sub.$this->_root_dn;
			$r2 =& $this->_lc->add($new_object, $this->_ent, $this->_pos);
			$r->merge_traces($r2);
			if ($r->has_error())
				$r>add_event('lms-api_objaddse');
			else
				$this->_reset_changed_flags();
		}

		return $r;
	}

	/**
	* Fetch every Attributs of the Object for the user
	*
	* @access	public
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &fetch_all_attrs()
	{
		$r =& $this->_fetch_all_attrs();
		if (!$r->has_error())
			$this->_entry_to_attrs($this->_pos, &$r);

		return $r;
	}

	/**
	* Fetch every Attributs of the Object (internaly only)
	*
	* @access	private
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_fetch_all_attrs()
	{
		$r =& $this->_tracemgr->create_trace();

		if ($this->_ent_get_count() == 0)
			$r->add_event('lms-api_objfetchaano');

		if (!$r->has_error())
		{
			$ent =& new DftkLdapEntries(&$this->_tracemgr);

			$r2 =& $this->_lc->search($this->_dn, "objectClass=*", $ent);
			$r->merge_traces($r2);
			if (!$r->has_error())
			{
				$r3 =& $ent->get_count();
				$nb = $r3->get_result('count');
				if ($nb==0)
					$r->add_event('lms-api_objfetchaanf');
				else
				{
					$r2 =& $ent->get_entry(0);
					$tab =& $r2->get_result('entry');
					$r2 =& $this->_ent->get_entry($this->_pos);
					$tab2 =& $r2->get_result('entry');
					$tab2 = $tab;
				}
			}
			else
				$r->add_event('lms-api_objfetchaase');
		}

		return $r;
	}

	/**
	* Check if required Attributs are set (CAN BE OVERLOADED, BUT MUST BE CALLED BY THE OVERLOAD METHOD)
	*
	* @access	private
	* @return	object DftkDaTrace	$r			Trace
	*/
	function &_check_changes($mod = array())
	{
		$r =& $this->_tracemgr->create_trace();

		foreach ($this->_attrs as $key => $val)
		{
			if ((count($mod)==0) || in_array($key, $mod))
			{
				if ($this->_attrs[$key]['needed'] && !$this->_attrs[$key]['value'])
					$r->add_event('lms-api_objchkchgan', $key);
			}
		}

		return $r;
	}

	/**
	* Do what need to be done before applying changes (CAN BE OVERLOADED)
	*
	* @access	private
	*/
	function _pre_commit_changes()
	{
		;
	}

	/**
	* Return a sub DN if needed (CAN BE OVERLOADED)

	*
	* @access	public
	* @return	string
	*/
	function _get_sub_dn()
	{
		return "";
	}

	/**
	* Do Object need a renaming before saving changes ? (CAN BE OVERLOADED)

	*
	* @access	private
	* @return	boolean
	*/
	function _need_renaming()
	{
		return false;
	}

}

?>
